"""
templatelib: Tool for generating files from templates. Intended for making websites.

Copyright 2003 Graham King

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

__version__ = 0.2

# Imports
import os
import os.path
import shutil
import glob

# Dictionary of what to map the keys in the pages to. So if this has { "key": "value" } and in a page you have 'Here is the <!%% key %%> thing' the output will produce 'Here is the value thing'
GLOBAL_VARS = {} 

# Dictionary of what to replace the keys in the pages with. Like GLOBAL_VARS but the value here is a filename and the whole file is read in and inserted where the key was
GLOBAL_FILES = {} 

# Where we are deploying FROM. All the templates and names in 'files' dictionary are asumed to be relative to here. This should not end with a slash
srcdir = ''

# Where we are deploying TO. All the generated files will be under this hierarchy. The 'target' are relative to here. This should not end with a slash
dstdir = ''

# Array of Page objects. What we will use to produce the output
pages = []

# The log produced as we generate files and the different log levels
logdata = [] 
LOG_DEBUG = 0 
LOG_INFO = 1 
LOG_ERROR = 2 
LOG_FATAL = 3 
loglevel = LOG_INFO
loglevelnames = {
	LOG_DEBUG: "DEBUG",
	LOG_INFO: "INFO",
	LOG_ERROR: "ERROR",
	LOG_FATAL: "FATAL" }

# Maximum number of passes we can make over a single file. This is to prevent loops
MAX_ITERATIONS = 5

#
# Page class: Holds the data needed for a page
#
class Page:
	def __init__(self, templateParam, targetParam):

		# The file to generate
		self.target = targetParam 

		# The template to use to generate it
		self.template = templateParam 

		# Local version of GLOBAL_VARS. This is searched before the global
		self.vars = {} 

		# Local version of GLOBAL_FILES. This is searched before the global
		self.files = {} 

"""
Iterates through all the pages generating them
"""
def generateall():

	if not os.path.exists(srcdir):
		log("Source directory '"+ srcdir + \
			"' does not exists. Please create it. Aborting.", LOG_FATAL)

	for page in pages:

		generate(page)

	print '\n'.join(logdata)

"""
Turns a Page object into an actual file
"""
def generate(page):


	(directory, filename) = os.path.split(page.target)

	if len(directory) != 0 and not os.path.exists(directory):
		os.mkdirs(directory)

	fulloutputname = dstdir + os.sep + page.target
	fulltemplatename = srcdir + os.sep + page.template

	log("Processing template '"+ fulltemplatename + \
		"' to target '"+ fulloutputname +"'")

	if not os.path.exists(fulltemplatename):
		log("File '"+ fulltemplatename +"' not found. Skipping.", LOG_ERROR)
		return


	shutil.copy(fulltemplatename, fulloutputname)
	log("Copied '"+ fulltemplatename +"' to '"+ fulloutputname +"'", LOG_DEBUG)

	complete(fulloutputname, page)

#
# Replaces all the tags in the file
#
def complete(filename, page):

	# Number of passes we have made over this file
	iterations = 0

	# Some files may need several pases
	while iterations == 0 or (haveFoundTag and iterations < MAX_ITERATIONS):	

		log("Completing '"+ filename +"'", LOG_DEBUG)
		
		haveFoundTag = 0
		iterations += 1
	
		input = open(filename, "rt")
		inputlines = input.readlines()
		input.close()
	
		outputlines = []
	
		for line in inputlines:
	
			startTagIndex = line.find("<!%%")
	
			if startTagIndex == -1:
	
				outputlines.append(line)
	
			else:
				haveFoundTag = 1
	
				endTagIndex = line.find("%%>", startTagIndex)
	
				tagname = line[startTagIndex + 4 : endTagIndex - 1].strip()
				log("Found tag '"+ tagname +"'", LOG_DEBUG)
	
				newline = line[:startTagIndex] + \
						replacement(tagname, page) + \
							line[endTagIndex + 3 :]
	
				outputlines.append( newline )
	
		output = open(filename, "wt")
		output.write( ''.join(outputlines) )
		output.close()

	if iterations == MAX_ITERATIONS:
		log("Stopped working on '"+ filename +"' after "+ \
			str(iterations) +". There may be a circular reference "+ \
			"(a including b which includes a) or you may need to "+ \
			"increase MAX_ITERATIONS", LOG_ERROR)

# Finds and returns the appropriate replacement string for the given tag
# Search order is page vars, page files, global vars, global files.
def replacement(tagname, page):

	if len(tagname) == 0:
		log("Tagname is empty. Returning empty string.", LOG_ERROR)

		return ""


	# Try the page's vars
	if page.vars.has_key(tagname):
		log("Tag '"+ tagname +"' found in page vars. "+ \
			"Contents: '"+ page.vars[tagname] +"'", LOG_DEBUG)

		return page.vars[tagname]

	# Try the page files
	elif page.files.has_key(tagname):
		log ("Tag '"+ tagname +"' found in page files. "+ \
			"Filename: '"+ page.files[tagname] +"'", LOG_DEBUG)

		file = open( page.files[tagname] )
		contents = file.readlines()
		file.close()

		return ''.join(contents)
	
	# Try the global vars
	elif GLOBAL_VARS.has_key(tagname):
		log("Tag '"+ tagname +"' found in global vars. "+ \
			"Contents: '"+ GLOBAL_VARS[tagname] +"'", LOG_DEBUG)

		return GLOBAL_VARS[tagname]

	# Try the global files
	elif GLOBAL_FILES.has_key(tagname):
		log ("Tag '"+ tagname +"' found in global files. "+ \
			"Filename: '"+ GLOBAL_FILES[tagname] +"'", LOG_DEBUG)

		file = open( GLOBAL_FILES[tagname] )
		contents = file.readlines()
		file.close()

		return ''.join(contents)
	else:
		log("No replacement found for tagname '"+ tagname + \
			"'. Using empty string.")

		return ""


def log(string, level=LOG_INFO):

	if level >= loglevel:
		logdata.append( loglevelnames[level] +": "+ string)

# Convenience method for deploying static resources. 
# Takes source and destination filenams. Accepts wildcards. 
# src can be a directory, but in that case dst must also be one.
# dst can be a directory even if src is not, but it must already exist (use makedirs). 
# If relative is true (1), templatelib.srcdir is prepended to src and templatelib.dstdir is prepended to dst.

def copy(src, dst, relative=0):

	if relative:
		# The replace is to prevent any duplicate slashes
		src = (srcdir + os.sep + src).replace(os.sep + os.sep, os.sep) 
		dst = (dstdir + os.sep + dst).replace(os.sep + os.sep, os.sep)

		if not os.path.exists(dstdir):
			makedirs(dstdir)

	log("Copying '"+ src +"' to '"+ dst +"'")

	# Whole directory copy

	if os.path.isdir(src):

		if not dst.endswith( os.sep ):
			dst += os.sep 

		if os.path.exists(dst):
			shutil.rmtree(dst)

		shutil.copytree(src, dst)
		return

	# File copy

	srcfiles = glob.glob(src)
	
	for filename in srcfiles:

		shutil.copy(filename, dst)

# Creates the directory given. Creates any necessary intermediary directories too
# If relative is true the templatelib.dstdir is prepended to the directory given
def makedirs(directory, relative=0):

	if relative:
		directory = (dstdir + os.sep + directory).replace(os.sep + os.sep, os.sep)

	if not os.path.exists(directory):
		os.makedirs(directory)
		log("Created directory '"+ directory +"'")


